@@ -2,7 +2,7 @@
 //                          Package   : omniNames
 // NamingContext_i.cc       Author    : Tristan Richardson (tjr)
 //
-//    Copyright (C) 1997 Olivetti & Oracle Research Laboratory
+//    Copyright (C) 1997-1999 AT&T Laboratories Cambridge
 //
 //  This file is part of omniNames.
 //
@@ -27,6 +27,7 @@
 #include <NamingContext_i.h>
 #include <ObjectBinding.h>
 #include <BindingIterator_i.h>
+#include <omniORB3/omniURI.h>
 
 #ifdef DEBUG_NC
 #define DB(x) x
@@ -42,30 +43,17 @@ NamingContext_i* NamingContext_i::tailContext = (NamingContext_i*)0;
 //
 // Ctor.
 //
-
-#if defined(__WIN32__) && defined(_MSC_VER)
-
-// Work-around MSVC++ 4.2 nested class bug
-typedef CosNaming::_sk_NamingContext CosNaming__sk_NamingContext;
-
-NamingContext_i::NamingContext_i(CORBA::BOA_ptr boa,
-				 const omniORB::objectKey& k, log* l)
-  : CosNaming__sk_NamingContext(k), redolog(l)
-
-#else
-
-NamingContext_i::NamingContext_i(CORBA::BOA_ptr boa,
-				 const omniORB::objectKey& k, log* l)
-  : CosNaming::_sk_NamingContext(k), redolog(l)
-
-#endif
+NamingContext_i::NamingContext_i(PortableServer::POA_ptr poa,
+				 const PortableServer::ObjectId& id,
+				 omniNameslog* l)
+  : redolog(l), nc_poa(poa)
 {
   headBinding = tailBinding = (ObjectBinding*)0;
   size = 0;
 
   WriterLock w(lock);
 
-  redolog->create(k);
+  redolog->create(id);
 
   prev = tailContext;
   next = (NamingContext_i*)0;
@@ -76,7 +64,7 @@ NamingContext_i::NamingContext_i(CORBA::BOA_ptr boa,
     headContext = this;
   }
 
-  _obj_is_ready(boa);
+  poa->activate_object_with_id(id, this);
 }
 
 
@@ -87,12 +75,15 @@ NamingContext_i::NamingContext_i(CORBA::BOA_ptr boa,
 CosNaming::NamingContext_ptr
 NamingContext_i::new_context()
 {
-  omniORB::objectKey k;
-  omniORB::generateNewKey(k);
+  CORBA::Object_var ref = the_poa->create_reference(
+				    CosNaming::NamingContext::_PD_repoId);
+  PortableServer::ObjectId_var id = the_poa->reference_to_id(ref);
 
-  NamingContext_i* nc = new NamingContext_i(_boa(), k, redolog);
+  NamingContext_i* nc = new NamingContext_i(the_poa, id, redolog);
+  CosNaming::NamingContext_ptr ncref = nc->_this();
+  nc->_remove_ref();
 
-  return nc->_this();
+  return ncref;
 }
 
 
@@ -435,7 +426,8 @@ NamingContext_i::destroy()
   CosNaming::NamingContext_var nc = _this();
   redolog->destroy(nc);
 
-  _dispose();
+  PortableServer::ObjectId_var id = nc_poa->servant_to_id(this);
+  nc_poa->deactivate_object(id);
 }
 
 
@@ -444,8 +436,8 @@ NamingContext_i::destroy()
 //
 
 void
-NamingContext_i::list(CORBA::ULong how_many, CosNaming::BindingList*& bl,
-		      CosNaming::BindingIterator_ptr& bi)
+NamingContext_i::list(CORBA::ULong how_many, CosNaming::BindingList_out bl,
+		      CosNaming::BindingIterator_out bi)
 {
   lock.readerIn();
 
@@ -470,11 +462,12 @@ NamingContext_i::list(CORBA::ULong how_many, CosNaming::BindingList*& bl,
 
   lock.readerOut();
 
-  BindingIterator_i* bii = new BindingIterator_i(_boa(), all);
+  BindingIterator_i* bii = new BindingIterator_i(the_poa, all);
 
   bi = bii->_this();
+  bii->_remove_ref();
 
-  if (CORBA::is_nil(bi)) {
+  if (CORBA::is_nil(bi.ptr())) {
     cerr << "couldn't narrow binding iterator" << endl;
     return;
   }
@@ -508,3 +501,33 @@ NamingContext_i::~NamingContext_i()
   while (headBinding)
     delete headBinding;
 }
+
+
+//
+// CosNaming::NamingContextExt operations
+//
+
+char*
+NamingContext_i::to_string(const CosNaming::Name& name)
+{
+  return omniURI::nameToString(name);
+}
+
+CosNaming::Name*
+NamingContext_i::to_name(const char* sn)
+{
+  return omniURI::stringToName(sn);
+}
+
+char*
+NamingContext_i::to_url(const char* addr, const char* sn)
+{
+  return omniURI::addrAndNameToURI(addr, sn);
+}
+
+CORBA::Object_ptr
+NamingContext_i::resolve_str(const char* sn)
+{
+  CosNaming::Name_var name = omniURI::stringToName(sn);
+  return resolve(name);
+}
