@@ -2,7 +2,7 @@
 //                          Package   : omniNames
 // log.cc                   Author    : Tristan Richardson (tjr)
 //
-//    Copyright (C) 1997 Olivetti & Oracle Research Laboratory
+//    Copyright (C) 1997-1999 AT&T Laboratories Cambridge
 //
 //  This file is part of omniNames.
 //
@@ -36,6 +36,7 @@
 #endif
 #include <NamingContext_i.h>
 #include <ObjectBinding.h>
+#include <INSMapper.h>
 #include <log.h>
 #include <iomanip.h>
 
@@ -63,6 +64,22 @@
 #include <sys/param.h>
 #endif
 
+#ifndef O_SYNC
+#ifdef  O_FSYNC              // FreeBSD 3.2 does not have O_SYNC???
+#define O_SYNC O_FSYNC
+#endif
+#endif
+
+#if defined(__SUNPRO_CC) && __SUNPRO_CC >= 0x500
+#define USE_STREAM_OPEN
+#define OPEN(name,mode,perm) open(name,mode,perm)
+#elif defined(__KCC)
+#define USE_STREAM_OPEN
+#define OPEN(name,mode,perm) open(name,mode)
+#elif defined(__GNUG__) && __GNUG__ >= 3
+#define USE_STREAM_OPEN
+#define OPEN(name,mode,perm) open(name,mode)
+#endif
 
 #ifdef _NO_STRDUP
 
@@ -80,6 +97,15 @@ strdup (char* str)
 }
 #endif  // _NO_STRDUP
 
+
+static
+inline void reallyFlush(ofstream& f) {
+    f.flush();
+#ifdef needsExplicitFsync
+    ::fsync(f.rdbuf()->fd());
+#endif
+}
+
 extern void usage();
 
 
@@ -115,17 +141,18 @@ timestamp ts;
 
 
 //
-// Constructor for class log.  There will normally be only one instance of this
-// class.  Unfortunately the initialisation of the class cannot be completed in
-// the constructor - the rest is done in the init() member function.  This is
-// because the main program cannot give us pointers to the ORB and the BOA
-// until it has worked out which port to listen on, but it normally finds out
-// the port from the log file.  So in this constructor we initialise as much as
-// we can, find out the port from the log file if there is one and return.
-// Then the main program initialises the ORB and the BOA and calls init().
+// Constructor for class omniNameslog.  There will normally be only one
+// instance of this class.  Unfortunately the initialisation of the class
+// cannot be completed in the constructor - the rest is done in the init()
+// member function.  This is because the main program cannot give us
+// pointers to the ORB and the BOA until it has worked out which port to
+// listen on, but it normally finds out the port from the log file.  So in
+// this constructor we initialise as much as we can, find out the port from
+// the log file if there is one and return.  Then the main program
+// initialises the ORB and the BOA and calls init().
 //
 
-log::log(int& p) : port(p)
+omniNameslog::omniNameslog(int& p,char* logdir) : port(p)
 {
   startingUp = 1;
   checkpointNeeded = 1;
@@ -137,9 +164,7 @@ log::log(int& p) : port(p)
   struct stat sb;
 #endif
 
-  char* logdir;
-
-  if ((logdir = getenv(LOGDIR_ENV_VAR)) == NULL)
+  if (!logdir && (logdir = getenv(LOGDIR_ENV_VAR)) == NULL)
     logdir = strdup(DEFAULT_LOGDIR);
 
 #if !defined(__WIN32__) && !defined(__VMS)
@@ -165,6 +190,16 @@ log::log(int& p) : port(p)
 			   + strlen(un.nodename) + 1];
   sprintf(logname, "%s/omninames-%s", logdir, un.nodename);
 #else
+
+  // Apparently on some AIX versions, MAXHOSTNAMELEN is too small (32) to
+  // reflect the true size a hostname can be. Check and fix the value.
+#ifndef MAXHOSTNAMELEN
+#define MAXHOSTNAMELEN 256
+#elif   MAXHOSTNAMELEN < 64
+#undef  MAXHOSTNAMELEN
+#define MAXHOSTNAMELEN 256
+#endif
+
   char hostname[MAXHOSTNAMELEN+1];
 
   if (gethostname(hostname, MAXHOSTNAMELEN) < 0) {
@@ -309,10 +344,13 @@ log::log(int& p) : port(p)
 
 
 void
-log::init(CORBA::ORB_ptr o, CORBA::BOA_ptr b)
+omniNameslog::init(CORBA::ORB_ptr          the_orb,
+		   PortableServer::POA_ptr the_poa,
+		   PortableServer::POA_ptr the_ins_poa)
 {
-  orb = o;
-  boa = b;
+  orb     = the_orb;
+  poa     = the_poa;
+  ins_poa = the_ins_poa;
 
   if (firstTime) {
 
@@ -324,29 +362,37 @@ log::init(CORBA::ORB_ptr o, CORBA::BOA_ptr b)
     cerr << ts.t() << "Starting omniNames for the first time." << endl;
 
     try {
-#ifdef __WIN32__
-      int fd = _open(active, O_WRONLY | O_CREAT | O_TRUNC, _S_IWRITE);
+#ifdef USE_STREAM_OPEN
+      logf.OPEN(active,ios::out|ios::trunc,0666);
+      if (!logf)
+	throw IOError();
 #else
+#  ifdef __WIN32__
+      int fd = _open(active, O_WRONLY | O_CREAT | O_TRUNC, _S_IWRITE);
+#  else
       int fd = open(active, O_WRONLY | O_CREAT | O_TRUNC | O_SYNC, 0666);
-#endif
-
+#  endif
       if (fd < 0)
 	throw IOError();
       logf.attach(fd);
+#endif
 
       putPort(port, logf);
+      reallyFlush(logf);
 
-      omniORB::objectKey k;
-      omniORB::generateNewKey(k);
+      {
+	PortableServer::ObjectId_var refid =
+	  PortableServer::string_to_ObjectId("NameService");
 
-      putCreate(k, logf);
+	putCreate(refid, logf);
+      }
 
       logf.close();
       if (!logf)
 	throw IOError();
 
 // a bug in sparcworks C++ means that the fd doesn't get closed.
-#if defined(__sunos__) && defined(__SUNPRO_CC)
+#if defined(__sunos__) && defined(__SUNPRO_CC) && __SUNPRO_CC < 0x500
       if (close(fd) < 0)
 	throw IOError();
 #endif
@@ -431,6 +477,17 @@ log::init(CORBA::ORB_ptr o, CORBA::BOA_ptr b)
   CosNaming::NamingContext_ptr rootContext
     = NamingContext_i::headContext->_this();
 
+  {
+    // Check to see if we need an INS forwarding agent
+    omniObjKey key;
+    rootContext->_getTheKey(key);
+
+    if (strncmp((const char*)key.key(), "NameService", 11)) {
+      cerr << ts.t() << "(Pre-INS log file)" << endl;
+      new INSMapper(the_ins_poa, rootContext);
+    }
+  }
+
   char* p = orb->object_to_string(rootContext);
   cerr << ts.t() << "Root context is " << p << endl;
   // Now use the backdoor to tell the bootstrap agent in this
@@ -441,11 +498,19 @@ log::init(CORBA::ORB_ptr o, CORBA::BOA_ptr b)
 
   CORBA::release(rootContext);	// dispose of the object reference
 
-#ifdef __WIN32__
-  int fd = _open(active, O_WRONLY | O_APPEND);
+#ifdef USE_STREAM_OPEN
+  logf.OPEN(active,ios::out|ios::app,0666);
+  if (!logf) {
+    cerr << ts.t() << "Error: cannot open log file '" << active
+	 << "' for writing." << endl;
+    exit(1);
+  }
 #else
+#  ifdef __WIN32__
+  int fd = _open(active, O_WRONLY | O_APPEND);
+#  else
   int fd = open(active, O_WRONLY | O_APPEND | O_SYNC);
-#endif
+#  endif
 
   if (fd < 0) {
     cerr << ts.t() << "Error: cannot open log file '" << active
@@ -453,6 +518,7 @@ log::init(CORBA::ORB_ptr o, CORBA::BOA_ptr b)
     exit(1);
   }
   logf.attach(fd);
+#endif
 
   startingUp = 0;
 
@@ -467,11 +533,12 @@ log::init(CORBA::ORB_ptr o, CORBA::BOA_ptr b)
 
 
 void
-log::create(const omniORB::objectKey& key)
+omniNameslog::create(const PortableServer::ObjectId& id)
 {
   if (!startingUp) {
     try {
-      putCreate(key, logf);
+      putCreate(id, logf);
+      reallyFlush(logf);
     } catch (IOError& ex) {
       cerr << ts.t() << flush;
       perror("I/O error writing log file");
@@ -482,11 +549,12 @@ log::create(const omniORB::objectKey& key)
 }
 
 void
-log::destroy(CosNaming::NamingContext_ptr nc)
+omniNameslog::destroy(CosNaming::NamingContext_ptr nc)
 {
   if (!startingUp) {
     try {
       putDestroy(nc, logf);
+      reallyFlush(logf);
     } catch (IOError& ex) {
       cerr << ts.t() << flush;
       perror("I/O error writing log file");
@@ -497,12 +565,13 @@ log::destroy(CosNaming::NamingContext_ptr nc)
 }
 
 void
-log::bind(CosNaming::NamingContext_ptr nc, const CosNaming::Name& n,
+omniNameslog::bind(CosNaming::NamingContext_ptr nc, const CosNaming::Name& n,
 	  CORBA::Object_ptr obj, CosNaming::BindingType t)
 {
   if (!startingUp) {
     try {
       putBind(nc, n, obj, t, logf);
+      reallyFlush(logf);
     } catch (IOError& ex) {
       cerr << ts.t() << flush;
       perror("I/O error writing log file");
@@ -513,11 +582,12 @@ log::bind(CosNaming::NamingContext_ptr nc, const CosNaming::Name& n,
 }
 
 void
-log::unbind(CosNaming::NamingContext_ptr nc, const CosNaming::Name& n)
+omniNameslog::unbind(CosNaming::NamingContext_ptr nc, const CosNaming::Name& n)
 {
   if (!startingUp) {
     try {
       putUnbind(nc, n, logf);
+      reallyFlush(logf);
     } catch (IOError& ex) {
       cerr << ts.t() << flush;
       perror("I/O error writing log file");
@@ -529,7 +599,7 @@ log::unbind(CosNaming::NamingContext_ptr nc, const CosNaming::Name& n)
 
 
 void
-log::checkpoint(void)
+omniNameslog::checkpoint(void)
 {
   if (!checkpointNeeded) {
     // cerr << ts.t() << "No checkpoint needed." << endl;
@@ -547,14 +617,24 @@ log::checkpoint(void)
   NamingContext_i::lock.readerIn();
 
   ofstream ckpf;
-
-#ifdef __WIN32__
-  int fd = _open(checkpt, O_WRONLY | O_CREAT | O_TRUNC, _S_IWRITE);
-#else
-  int fd = open(checkpt, O_WRONLY | O_CREAT | O_TRUNC | O_SYNC, 0666);
-#endif
+  int fd = -1;
 
   try {
+
+#ifdef USE_STREAM_OPEN
+    ckpf.OPEN(checkpt,ios::out|ios::trunc,0666);
+    if (!ckpf) {
+      cerr << ts.t() << "Error: cannot open checkpoint file '"
+	   << checkpt << "' for writing." << endl;
+      throw IOError();
+    }
+#else
+#  ifdef __WIN32__
+    fd = _open(checkpt, O_WRONLY | O_CREAT | O_TRUNC, _S_IWRITE);
+#  else
+    fd = open(checkpt, O_WRONLY | O_CREAT | O_TRUNC | O_SYNC, 0666);
+#  endif
+
     if (fd < 0) {
       cerr << ts.t() << "Error: cannot open checkpoint file '"
 	   << checkpt << "' for writing." << endl;
@@ -562,13 +642,15 @@ log::checkpoint(void)
     }
 
     ckpf.attach(fd);
+#endif
 
     putPort(port, ckpf);
 
     NamingContext_i* nci;
 
     for (nci = NamingContext_i::headContext; nci; nci = nci->next) {
-      putCreate(nci->_key(), ckpf);
+      PortableServer::ObjectId_var id = nci->PR_id();
+      putCreate(id, ckpf);
     }
 
     for (nci = NamingContext_i::headContext; nci; nci = nci->next) {
@@ -583,7 +665,7 @@ log::checkpoint(void)
       throw IOError();
 
 // a bug in sparcworks C++ means that the fd doesn't get closed.
-#if defined(__sunos__) && defined(__SUNPRO_CC)
+#if defined(__sunos__) && defined(__SUNPRO_CC) && __SUNPRO_CC < 0x500
     if (close(fd) < 0)
       throw IOError();
 #endif
@@ -594,7 +676,7 @@ log::checkpoint(void)
     cerr << "Abandoning checkpoint" << endl;
     ckpf.close();
 // a bug in sparcworks C++ means that the fd doesn't get closed.
-#if defined(__sunos__) && defined(__SUNPRO_CC)
+#if defined(__sunos__) && defined(__SUNPRO_CC) && __SUNPRO_CC < 0x500
     close(fd);
 #endif
     NamingContext_i::lock.readerOut();
@@ -610,7 +692,7 @@ log::checkpoint(void)
   cerr << ts.t() << "Checkpointing Phase 2: Commit." << endl;
 
 // a bug in sparcworks C++ means that the fd doesn't get closed.
-#if defined(__sunos__) && defined(__SUNPRO_CC)
+#if defined(__sunos__) && defined(__SUNPRO_CC) && __SUNPRO_CC < 0x500
   close(logf.rdbuf()->fd());
 #endif
 
@@ -662,11 +744,19 @@ log::checkpoint(void)
   }
 #endif
 
-#ifdef __WIN32__
-  fd = _open(active, O_WRONLY | O_APPEND);
+#ifdef USE_STREAM_OPEN
+  logf.OPEN(active,ios::out|ios::app,0666);
+  if (!logf) {
+    cerr << ts.t() << "Error: cannot open log file '" << active
+	 << "' for writing." << endl;
+    exit(1);
+  }
 #else
+#  ifdef __WIN32__
+  fd = _open(active, O_WRONLY | O_APPEND);
+#  else
   fd = open(active, O_WRONLY | O_APPEND | O_SYNC);
-#endif
+#  endif
 
   if (fd < 0) {
     cerr << ts.t() << "Error: cannot open new log file '" << active
@@ -674,6 +764,7 @@ log::checkpoint(void)
     exit(1);
   }
   logf.attach(fd);
+#endif
 
   NamingContext_i::lock.readerOut();
 
@@ -684,9 +775,9 @@ log::checkpoint(void)
 
 
 void
-log::putPort(int p, ostream& file)
+omniNameslog::putPort(int p, ostream& file)
 {
-  file << "port " << p << '\n' << flush;
+  file << "port " << p << '\n';
   if (!file) throw IOError();
 }
 
@@ -698,7 +789,7 @@ log::putPort(int p, ostream& file)
 //
 
 void
-log::getPort(istream& file)
+omniNameslog::getPort(istream& file)
 {
   char* str;
 
@@ -725,41 +816,49 @@ log::getPort(istream& file)
 
 
 void
-log::putCreate(const omniORB::objectKey& key, ostream& file)
+omniNameslog::putCreate(const PortableServer::ObjectId& id, ostream& file)
 {
   file << "create ";
-  putKey(key, file);
-  file << '\n' << flush;
+  putKey(id, file);
+  file << '\n';
   if (!file) throw IOError();
 }
 
 
 void
-log::getCreate(istream& file)
+omniNameslog::getCreate(istream& file)
 {
   //
   // Argument to "create" is the object key of the naming context.
   //
 
-  omniORB::objectKey k;
-  getKey(k, file);
-  NamingContext_i* rc = new NamingContext_i(boa, k, this);
+  PortableServer::ObjectId id;
+  NamingContext_i*         rc;
+
+  getKey(id, file);
+
+  if (id.length() == 4) // SYS_ASSIGNED_ID_SIZE
+    rc = new NamingContext_i(poa, id, this);
+  else
+    rc = new NamingContext_i(ins_poa, id, this);
+
+  rc->_remove_ref();
 }
 
 
 void
-log::putDestroy(CosNaming::NamingContext_ptr nc, ostream& file)
+omniNameslog::putDestroy(CosNaming::NamingContext_ptr nc, ostream& file)
 {
   file << "destroy ";
   CORBA::String_var s = orb->object_to_string(nc);
   putString(s, file);
-  file << '\n' << flush;
+  file << '\n';
   if (!file) throw IOError();
 }
 
 
 void
-log::getDestroy(istream& file)
+omniNameslog::getDestroy(istream& file)
 {
   //
   // Argument to "destroy" is NamingContext IOR.
@@ -789,7 +888,7 @@ log::getDestroy(istream& file)
 
 
 void
-log::putBind(CosNaming::NamingContext_ptr nc, const CosNaming::Name& n,
+omniNameslog::putBind(CosNaming::NamingContext_ptr nc, const CosNaming::Name& n,
 	     CORBA::Object_ptr obj, CosNaming::BindingType t, ostream& file)
 {
   file << "bind ";
@@ -805,13 +904,13 @@ log::putBind(CosNaming::NamingContext_ptr nc, const CosNaming::Name& n,
     file << " ncontext ";
   s = orb->object_to_string(obj);
   putString(s, file);
-  file << '\n' << flush;
+  file << '\n';
   if (!file) throw IOError();
 }
 
 
 void
-log::getBind(istream& file)
+omniNameslog::getBind(istream& file)
 {
   //
   // First arg is NamingContext IOR.
@@ -891,7 +990,7 @@ log::getBind(istream& file)
 
 
 void
-log::putUnbind(CosNaming::NamingContext_ptr nc, const CosNaming::Name& n,
+omniNameslog::putUnbind(CosNaming::NamingContext_ptr nc, const CosNaming::Name& n,
 	       ostream& file)
 {
   file << "unbind ";
@@ -901,13 +1000,13 @@ log::putUnbind(CosNaming::NamingContext_ptr nc, const CosNaming::Name& n,
   putString(n[0].id, file);
   file << ' ';
   putString(n[0].kind, file);
-  file << '\n' << flush;
+  file << '\n';
   if (!file) throw IOError();
 }
 
 
 void
-log::getUnbind(istream& file)
+omniNameslog::getUnbind(istream& file)
 {
   //
   // First arg is NamingContext IOR.
@@ -949,41 +1048,42 @@ log::getUnbind(istream& file)
 
 
 void
-log::putKey(const omniORB::objectKey& key, ostream& file)
+omniNameslog::putKey(const PortableServer::ObjectId& id, ostream& file)
 {
-  omniORB::seqOctets* os = omniORB::keyToOctetSequence(key);
   file << hex;
-  for (unsigned int i = 0; i < os->length(); i++) {
-    file << setfill('0') << setw(2) << (int)(*os)[i];
+  for (unsigned int i = 0; i < id.length(); i++) {
+#if !defined(__SUNPRO_CC) || __SUNPRO_CC < 0x500
+    file << setfill('0') << setw(2) << (int)id[i];
+#else
+    // Eventually, use the following for all standard C++ compilers
+    file << std::setfill('0') << std::setw(2) << (int)id[i];
+#endif
   }
   file << dec;
-  delete os;
 }
 
 
 void
-log::getKey(omniORB::objectKey& k, istream& file)
+omniNameslog::getKey(PortableServer::ObjectId& id, istream& file)
 {
   char* str;
   getFinalString(str, file);
 
   int l = strlen(str) / 2;
-  omniORB::seqOctets os(l);
-  os.length(l);
+  id.length(l);
   char* p = str;
   for (int i = 0; i < l; i++) {
     int n;
     sscanf(p,"%02x",&n);
-    os[i] = n;
+    id[i] = n;
     p += 2;
   }
-  k = omniORB::octetSequenceToKey(os);  
   delete [] str;
 }
 
 
 void
-log::putString(const char* str, ostream& file)
+omniNameslog::putString(const char* str, ostream& file)
 {
   for (int i = strlen(str); i > 0; i--, str++) {
     switch (*str) {
@@ -1016,7 +1116,7 @@ log::putString(const char* str, ostream& file)
 
 
 void
-log::getFinalString(char*& buf, istream& file)
+omniNameslog::getFinalString(char*& buf, istream& file)
 {
   if (getString(buf, file) != '\n')
     throw ParseError();
@@ -1024,14 +1124,14 @@ log::getFinalString(char*& buf, istream& file)
 }
 
 void
-log::getNonfinalString(char*& buf, istream& file)
+omniNameslog::getNonfinalString(char*& buf, istream& file)
 {
   if (getString(buf, file) != ' ')
     throw ParseError();
 }
 
 int
-log::getString(char*& buf, istream& file)
+omniNameslog::getString(char*& buf, istream& file)
 {
   int bufsz = 512;
   buf = new char[bufsz];
