@@ -2,7 +2,7 @@
 //                          Package   : catior
 // catior.cc                Author    : Eoin Carroll (ewc)
 //
-//    Copyright (C) 1997 Olivetti & Oracle Research Laboratory
+//    Copyright (C) 1997-1999 AT&T Laboratories Cambridge
 //
 //  This file is part of catior.
 //
@@ -26,8 +26,11 @@
 
 #include <iostream.h>
 #include <stdlib.h>
+#ifndef __WIN32__
+#include <unistd.h>
+#endif
 
-#include <omniORB2/CORBA.h>
+#include <omniORB3/CORBA.h>
 
 #ifndef Swap16
 #define Swap16(s) ((((s) & 0xff) << 8) | (((s) >> 8) & 0xff))
@@ -48,15 +51,16 @@
 
 static void usage(char* progname)
 {
-  cerr << "usage: " << progname << " [-x] <stringified IOR>" << endl;
-  cerr << "         flag:" << endl;
-  cerr << "               -x print object key as a hexadecimal value." << endl;
+  cerr << "usage: " << progname << " [-x] [-o] <stringified IOR>" << endl;
+  cerr << "  flag:" << endl;
+  cerr << "     -x print object key as a hexadecimal value." << endl;
+  cerr << "     -o print extra info for omniORB IOR." << endl;
 }
 
 
 #if defined(__WIN32__) || defined(__VMS) && __VMS_VER < 60200000
 
-// WIN32 doesn't have an implementation of getopt() - 
+// WIN32 doesn't have an implementation of getopt() -
 // supply a getopt() for this program:
 
 char* optarg;
@@ -71,7 +75,7 @@ getopt(int num_args, char* const* args, const char* optstring)
   char* buf_left = *(args+optind);
 
   if ((*buf_left != '-' && *buf_left != '/') || buf_left == NULL ) return EOF;
-  else if ((optind < (num_args-1)) && strcmp(buf_left,"-") == 0 && 
+  else if ((optind < (num_args-1)) && strcmp(buf_left,"-") == 0 &&
 	   strcmp(*(args+optind+1),"-") == 0)
     {
       optind+=2;
@@ -107,7 +111,7 @@ getopt(int num_args, char* const* args, const char* optstring)
 		}
 	    }
 	  else optind++;
-	  
+
 	  return buf_left[1];
 	}
     }
@@ -175,7 +179,7 @@ EncapStreamToProfile(const _CORBA_Unbounded_Sequence_Octet &s,
       throw CORBA::NO_MEMORY(0,CORBA::COMPLETED_NO);
     memcpy((void *)p.host,(void *)&(s[begin]),len);
   }
-    
+
   // align to CORBA::UShort
   begin = (end + 1) & ~(1);
   // s[begin] port number
@@ -223,6 +227,129 @@ EncapStreamToProfile(const _CORBA_Unbounded_Sequence_Octet &s,
 }
 
 
+#define POA_NAME_SEP            '\xff'
+#define TRANSIENT_SUFFIX_SEP    '\xfe'
+#define TRANSIENT_SUFFIX_SIZE   8
+
+
+static void
+print_key(_CORBA_Unbounded_Sequence__Octet& key, int hexflag)
+{
+  if (hexflag) {
+    // Output key in hexadecimal form.
+
+    cout << "0x";
+
+    for( unsigned j = 0; j < key.length(); j++ ) {
+      int v = (key[j] & 0xf0) >> 4;
+
+      if (v < 10)
+	cout << (char)('0' + v);
+      else
+	cout << (char)('a' + (v - 10));
+
+      v = key[j] & 0xf;
+
+      if (v < 10)
+	cout << (char)('0' + v);
+      else
+	cout << (char)('a' + (v - 10));
+    }
+
+    cout << "  (" << key.length()
+	 << " bytes)" << endl;
+  }
+  else {
+    // Output key as text
+
+    cout << "\"";
+
+    for( unsigned j = 0; j < key.length(); j++ ) {
+      if( (char) key[j] >= ' ' &&
+	  (char) key[j] <= '~')
+	cout << (char) key[j];
+      else
+	cout << ".";
+    }
+    cout << "\"" << endl;
+  }
+}
+
+
+static int
+get_poa_info(_CORBA_Unbounded_Sequence__Octet& key,
+	     _CORBA_Unbounded_Sequence__String& poas_out,
+	     int& transient_out,
+	     _CORBA_Unbounded_Sequence__Octet& id_out)
+{
+  const char* k = (const char*) key.NP_data();
+  int len = key.length();
+  const char* kend = k + len;
+
+  poas_out.length(1);
+  poas_out[0] = CORBA::string_dup("root");
+
+  if( *k != TRANSIENT_SUFFIX_SEP && *k != POA_NAME_SEP )  return 0;
+
+  while( k < kend && *k == POA_NAME_SEP ) {
+
+    k++;
+    const char* name = k;
+
+    while( k < kend && *k && *k != POA_NAME_SEP && *k != TRANSIENT_SUFFIX_SEP )
+      k++;
+
+    if( k == kend )  return 0;
+
+    char* nm = new char[k - name + 1];
+    memcpy(nm, name, k - name);
+    nm[k - name] = '\0';
+    poas_out.length(poas_out.length() + 1);
+    poas_out[poas_out.length() - 1] = nm;
+  }
+
+  if( k == kend )  return 0;
+
+  transient_out = 0;
+  if( *k == TRANSIENT_SUFFIX_SEP ) {
+    transient_out = 1;
+    k += TRANSIENT_SUFFIX_SIZE + 1;
+  }
+  if( k >= kend || *k )  return 0;
+  k++;
+
+  id_out.length(kend - k);
+  memcpy(id_out.NP_data(), k, kend - k);
+
+  return 1;
+}
+
+
+static void
+print_omni_key(_CORBA_Unbounded_Sequence__Octet& key, int hexflag)
+{
+  _CORBA_Unbounded_Sequence__String poas;
+  int is_transient;
+  _CORBA_Unbounded_Sequence__Octet id;
+
+  if( get_poa_info(key, poas, is_transient, id) ) {
+    cout << "POA(" << (char*)poas[0];
+    for( unsigned i = 1; i < poas.length(); i++ )
+      cout << '/' << (char*)poas[i];
+    cout << ") ";
+  }
+  else {
+    if( key.length() != sizeof(omniOrbBoaKey) ) {
+      cout << "Object key not recognised as from omniORB." << endl;
+      exit(-1);
+    }
+    cout << "BOA ";
+  }
+
+  print_key(id, hexflag);
+}
+
+
 #if !defined(__WIN32__)
 extern char* optarg;
 extern int optind;
@@ -231,141 +358,95 @@ extern int optind;
 
 int main(int argc, char* argv[])
 {
-  if (argc < 2) 
-    {
-      usage(argv[0]);
-      return 1;
-    }	
+  if (argc < 2) {
+    usage(argv[0]);
+    return 1;
+  }
 
 
   // Get options:
 
   int c;
   int hexflag = 0;
+  int omniflag = 0;
 
-  while((c = getopt(argc,argv,"x")) != EOF)
-    {
-      switch(c)
-	{
-	case 'x':
-	  hexflag = 1;
-	  break;
-	case '?':
-	case 'h':
-	  {
-	    usage(argv[0]);
-	    return 1;
-	  }
-	}
-    }
-	  
-  if (optind >= argc)
-    {
-      usage(argv[0]);
-      return 1;
+  while((c = getopt(argc,argv,"xo")) != EOF) {
+    switch(c) {
+    case 'x':
+      hexflag = 1;
+      break;
+    case 'o':
+      omniflag = 1;
+      break;
+    case '?':
+    case 'h':
+      {
+	usage(argv[0]);
+	return 1;
+      }
     }
+  }
 
-  
-  char* str_ior = argv[optind];
+  if (optind >= argc) {
+    usage(argv[0]);
+    return 1;
+  }
 
-  CORBA::Char* repoID;
+
+  _CORBA_Char* str_ior = (CORBA::Char*)argv[optind];
+
+  _CORBA_Char* repoID;
   IOP::TaggedProfileList* profiles;
 
-  try
-    {
-      IOP::EncapStrToIor((CORBA::Char*) str_ior, repoID, profiles);
-      if (*repoID == '\0' && profiles->length() == 0)
-	{
-	  cerr << "IOR is a nil object reference." << endl;
+  try {
+    IOP::EncapStrToIor(str_ior, repoID, profiles);
+
+    if (*repoID == '\0' && profiles->length() == 0) {
+      cout << "IOR is a nil object reference." << endl;
+    }
+    else {
+      cout << "Type ID: \"" << (char*) repoID << "\"" << endl;
+      cout << "Profiles:" << endl;
+
+      for (unsigned long count=0; count < profiles->length(); count++) {
+	cout << count+1 << ". ";
+
+	if ((*profiles)[count].tag == IOP::TAG_INTERNET_IOP) {
+	  IIOP::ProfileBody pBody;
+	  EncapStreamToProfile((*profiles)[count].profile_data, pBody);
+	  cout << "IIOP " << (int) pBody.iiop_version.major << "."
+	       << (int) pBody.iiop_version.minor << " ";
+	  cout << (char*) pBody.host << " " << pBody.port << " ";
+
+	  unsigned long j;
+
+	  if( omniflag )
+	    print_omni_key(pBody.object_key, hexflag);
+	  else
+	    print_key(pBody.object_key, hexflag);
+
 	}
-      else
-	{
-	  cerr << "Type ID: \"" << (char*) repoID << "\"" << endl;
-	  cerr << "Profiles:" << endl;
-	  
-	  for (long count=0; count < profiles->length(); count++)
-	    {
-	      cout << count+1 << ". ";
-	      
-	      if ((*profiles)[count].tag == IOP::TAG_INTERNET_IOP)
-		  {
-		    IIOP::ProfileBody pBody;
-		    EncapStreamToProfile((*profiles)[count].profile_data,pBody);
-		    cout << "IIOP " << (int) pBody.iiop_version.major << "."
-			 << (int) pBody.iiop_version.minor << " ";
-		    cout << (char*) pBody.host << " " << pBody.port << " ";
-
-
-		    unsigned long j;
-		    if (hexflag)
-		      {
-			// Output key in hexadecimal form.
-			
-			cout << "0x";
-			
-
-			for (j=0; j < (pBody.object_key).length(); j++) 
-			  {
-			    int v = (((pBody.object_key)[j] & 0xf0) >> 4);
-
-			    if (v < 10)
-			      cout << (char)('0' + v);
-			    else
-			      cout << (char)('a' + (v - 10));
-
-			    v = (((pBody.object_key)[j] & 0xf));
-
-			    if (v < 10)
-			      cout << (char)('0' + v);
-			    else
-			      cout << (char)('a' + (v - 10));
-			  }
-
-			cout << "  (" << (pBody.object_key).length() 
-			     << " bytes)" << endl;
-		      }
-		    else
-		      {
-			  // Output key as text
-
-			  cout << "\"";
-			  
-			  for(j=0; j < (pBody.object_key).length(); j++)
-			    {
-				if ((char) ((pBody.object_key)[j]) >= ' ' &&
-				    (char) ((pBody.object_key)[j]) <= '~')
-				         cout << (char)((pBody.object_key)[j]);
-				else
-				    cout << ".";
-			    }
-			  cout << "\"" << endl;
-		      }
-		  }
-	      else if ((*profiles)[count].tag == IOP::TAG_MULTIPLE_COMPONENTS)
-		  {
-		    cout << "Multiple Component Tag" << endl;
-		  }
-	      else
-		  {
-		    cout << "Unrecognised profile tag: " 
-		         << (int) ((*profiles)[count].tag) 
-			 << endl;
-		  }
-	    }
+	else if ((*profiles)[count].tag == IOP::TAG_MULTIPLE_COMPONENTS) {
+	  cout << "Multiple Component Tag" << endl;
 	}
+	else {
+	  cout << "Unrecognised profile tag: "
+	       << (int) ((*profiles)[count].tag)
+	       << endl;
+	}
+      }
     }
-  catch(CORBA::MARSHAL& ex)
-    {
-      cerr << "Invalid stringified IOR supplied." << endl;
-      cerr << "(Minor = " << ex.minor() << ")" << endl;
-      return -1;
-    }
-  catch(...)
-    {
-      cerr << "Exception while processing stringified IOR." << endl;
-      return -1;
-    }
-  
+  }
+  catch(CORBA::MARSHAL& ex) {
+    cerr << "Invalid stringified IOR supplied." << endl;
+    cerr << "(Minor = " << ex.minor() << ")" << endl;
+    return -1;
+  }
+  catch(...) {
+    cerr << "Exception while processing stringified IOR." << endl;
+    return -1;
+  }
+
   delete[] repoID;
   delete profiles;
   return 1;
