@@ -2,7 +2,7 @@
 //                          Package   : nameclt
 // nameclt.cc               Author    : Tristan Richardson (tjr)
 //
-//    Copyright (C) 1997 Olivetti & Oracle Research Laboratory
+//    Copyright (C) 1997-1999 AT&T Laboratories Cambridge
 //
 //  This file is part of nameclt.
 //
@@ -24,22 +24,18 @@
 
 #include <iostream.h>
 #include <stdlib.h>
-#include "omniORB2/Naming.hh"
+#include <omniORB3/Naming.hh>
+#include <omniORB3/omniURI.h>
 
 static int advanced = 0;
 static const char* command;
 
-static void GetName(int argc, char** argv, int& n, int argsAfter,
-		    const char* usageAfter, CosNaming::Name& name);
-
 static void
 usage(const char* progname)
 {
   cerr << "\nusage: " << progname;
 
-  if (advanced) {
-    cerr << " [-advanced]";
-  }
+  cerr << " [-advanced]";
 
   cerr << " [-ior <NameService-object-reference>] <operation>"
        << "\n\nwhere <operation> is one of:\n"
@@ -58,23 +54,20 @@ usage(const char* progname)
        << "\n resolve <object-name>"
        << "\n    (returns stringified IOR bound to specified name)";
 
-  if (advanced) {
-    cerr << "\n\nAdvanced operations:\n"
-	 << "\n bind_context <context-name> <stringified-context-IOR>"
-	 << "\n    (binds name to context)"
-	 << "\n rebind <object-name> <stringified-IOR>"
-	 << "\n    (binds name to object even if binding already exists)"
-	 << "\n rebind_context <context-name> <stringified-context-IOR>"
-	 << "\n    (binds name to context even if binding already exists)"
-	 << "\n new_context"
-	 << "\n    (returns stringified IOR for a new context)"
-	 << "\n destroy"
-	 << "\n    (destroys the naming context given with -ior flag)";
-  }
+  cerr << "\n\nAdvanced operations:\n"
+       << "\n bind_context <context-name> <stringified-context-IOR>"
+       << "\n    (binds name to context)"
+       << "\n rebind <object-name> <stringified-IOR>"
+       << "\n    (binds name to object even if binding already exists)"
+       << "\n rebind_context <context-name> <stringified-context-IOR>"
+       << "\n    (binds name to context even if binding already exists)"
+       << "\n new_context"
+       << "\n    (returns stringified IOR for a new context)"
+       << "\n destroy"
+       << "\n    (destroys the naming context given with -ior flag)";
 
-  cerr << "\n\n<object-name>,<context-name> :  name1_id name1_kind ..."
-       << " nameN_id nameN_kind\n"
-       << endl;
+  cerr << "\n\n<object-name>,<context-name> :  name1_id.kind/.../nameN_id.kind"
+       << "\n" << endl;
   exit(1);
 }
 
@@ -84,8 +77,7 @@ main(int argc, char **argv)
 {
   // Initialize the ORB, the object adaptor and the echo object:
 
-  CORBA::ORB_ptr orb = CORBA::ORB_init(argc,argv,"omniORB2");
-  CORBA::BOA_ptr boa = orb->BOA_init(argc,argv,"omniORB2_BOA");
+  CORBA::ORB_ptr orb = CORBA::ORB_init(argc,argv,"omniORB3");
 
   const char* ior = (const char*)0;
   int n;
@@ -146,7 +138,19 @@ main(int argc, char **argv)
     }
   }
 
-  rootContext = CosNaming::NamingContext::_narrow(initServ);
+  try {
+    rootContext = CosNaming::NamingContext::_narrow(initServ);
+  }
+  catch (CORBA::COMM_FAILURE& ex) {
+    cerr << "Caught a COMM_FAILURE when trying to validate the type of the "
+	 << "NamingContext." << endl;
+    exit(1);
+  }
+  catch(...) {
+    cerr << "Unexpected error when trying to narrow the NamingContext."
+	 << endl;
+    exit(1);
+  }
   if (CORBA::is_nil(rootContext)) {
     cerr << "NameService object reference was not a NamingContext." << endl;
     exit(1);
@@ -168,47 +172,57 @@ main(int argc, char **argv)
 
       CosNaming::NamingContext_var context;
 
-      if (argc == n) {
+      switch (argc-n) {
 
+      case 0:
 	context = rootContext;
+	break;
 
-      } else {
+      case 1:
+	{
+	  CosNaming::Name_var name = omniURI::stringToName(argv[n]);
 
-	CosNaming::Name name;
-	GetName(argc, argv, n, 0, "", name);
+	  if (name->length() == 0) {
+	    context = rootContext;
+	  } else {
+	    CORBA::Object_var obj = rootContext->resolve(name);
 
-	CORBA::Object_var obj = rootContext->resolve(name);
+	    context = CosNaming::NamingContext::_narrow(obj);
 
-	context = CosNaming::NamingContext::_narrow(obj);
-
-	if (CORBA::is_nil(context)) {
-	  cerr << "list: not a naming context" << endl;
-	  exit(1);
+	    if (CORBA::is_nil(context)) {
+	      cerr << "list: not a naming context" << endl;
+	      exit(1);
+	    }
+	  }
+	  break;
 	}
+
+      default:
+	cerr << "usage: " << argv[0] << " " << command
+	     << " name1_id.kind/.../nameN_id.kind" << endl;
+	exit(1);
       }
 
-      CosNaming::BindingList* bl = 0;
-      CosNaming::BindingIterator_ptr bi = 0;
-      int first_n = 5, repeat_n = 3;
+      CosNaming::BindingIterator_var bi;
+      CosNaming::BindingList_var bl;
+      CosNaming::Binding_var b;
 
-      context->list(first_n, bl, bi);
+      context->list(0, bl, bi);
 
-      do {
-	for (unsigned int i = 0; i < bl->length(); i++) {
-	  cerr << "(" << (char*)(*bl)[i].binding_name[0].id << ","
-	       << (char*)(*bl)[i].binding_name[0].kind << ") binding type "
-	       << (((*bl)[i].binding_type == CosNaming::nobject) ?
-		   "nobject" : "ncontext")
-	       << endl;
-	}
+      while (bi->next_one(b)) {
 
-	delete bl;
-	bl = 0;
+	CORBA::String_var sname = omniURI::nameToString(b->binding_name);
 
-      } while (bi->next_n(repeat_n, bl));
+	cout << (const char*)sname;
+
+	if (b->binding_type == CosNaming::ncontext)
+	  cout << "/";
+
+	cout << endl;
+      }
 
       bi->destroy();
-      return 0;
+      goto done;
     }
 
 
@@ -219,14 +233,20 @@ main(int argc, char **argv)
 
     if (strcmp(command, "bind_new_context") == 0) {
 
-      CosNaming::Name name;
-      GetName(argc, argv, n, 0, "", name);
+      if (argc != n+1) {
+	cerr << "usage: " << argv[0] << " " << command
+	     << " name1_id.kind/.../nameN_id.kind" << endl;
+	exit(1);
+      }
 
-      CosNaming::NamingContext_var context = rootContext->bind_new_context(name);
+      CosNaming::Name_var name = omniURI::stringToName(argv[n]);
+
+      CosNaming::NamingContext_var context
+	= rootContext->bind_new_context(name);
 
       char* p = orb->object_to_string(context);
-      cerr << p << endl;
-      return 0;
+      cout << p << endl;
+      goto done;
     }
 
 
@@ -239,8 +259,13 @@ main(int argc, char **argv)
 
     if (strcmp(command, "remove_context") == 0) {
 
-      CosNaming::Name name;
-      GetName(argc, argv, n, 0, "", name);
+      if (argc != n+1) {
+	cerr << "usage: " << argv[0] << " " << command
+	     << " name1_id.kind/.../nameN_id.kind" << endl;
+	exit(1);
+      }
+
+      CosNaming::Name_var name = omniURI::stringToName(argv[n]);
 
       CORBA::Object_var obj = rootContext->resolve(name);
 
@@ -255,7 +280,7 @@ main(int argc, char **argv)
       context->destroy();
 
       rootContext->unbind(name);
-      return 0;
+      goto done;
     }
 
 
@@ -266,8 +291,13 @@ main(int argc, char **argv)
 
     if (strcmp(command, "bind") == 0) {
 
-      CosNaming::Name name;
-      GetName(argc, argv, n, 1, "objref", name);
+      if (argc != n+2) {
+	cerr << "usage: " << argv[0] << " " << command
+	     << " name1_id.kind/.../nameN_id.kind objref" << endl;
+	exit(1);
+      }
+
+      CosNaming::Name_var name = omniURI::stringToName(argv[n++]);
 
       CORBA::Object_var obj;
 
@@ -280,7 +310,7 @@ main(int argc, char **argv)
       }
 
       rootContext->bind(name, obj);
-      return 0;
+      goto done;
     }
 
 
@@ -291,26 +321,76 @@ main(int argc, char **argv)
 
     if (strcmp(command, "unbind") == 0) {
 
-      CosNaming::Name name;
-      GetName(argc, argv, n, 0, "", name);
+      if (argc != n+1) {
+	cerr << "usage: " << argv[0] << " " << command
+	     << " name1_id.kind/.../nameN_id.kind" << endl;
+	exit(1);
+      }
 
-      if (!advanced) {
+      CosNaming::Name_var name = omniURI::stringToName(argv[n]);
+
+      if (advanced) {
+
+	// Don't perform any safety checks for -advanced
+
+	rootContext->unbind(name);
+	goto done;
+      }
+
+      //
+      // Without -advanced, list parent context and check that binding type is
+      // nobject before allowing unbind.
+      //
+
+      CosNaming::NamingContext_var context;
+      int len = name->length();
+      CORBA::String_var id = name[len-1].id;
+      CORBA::String_var kind = name[len-1].kind;
+
+      if (len > 1) {
+	name->length(len-1);
 
 	CORBA::Object_var obj = rootContext->resolve(name);
 
-	CosNaming::NamingContext_var context
-	  = CosNaming::NamingContext::_narrow(obj);
+	context = CosNaming::NamingContext::_narrow(obj);
 
-	if (!CORBA::is_nil(context)) {
-	  cerr << "Error: unbind: can't unbind a naming context.\n"
-	       << "Use remove_context to remove an empty naming context."
-	       << endl;
+	if (CORBA::is_nil(context)) {
+	  cerr << "unbind: parent is not a naming context" << endl;
 	  exit(1);
 	}
+      } else {
+	context = rootContext;
       }
 
-      rootContext->unbind(name);
-      return 0;
+      CosNaming::BindingIterator_var bi;
+      CosNaming::BindingList_var bl;
+      CosNaming::Binding_var b;
+
+      context->list(0, bl, bi);
+
+      while (bi->next_one(b)) {
+
+	if (strcmp(b->binding_name[0].id, id) == 0 &&
+	    strcmp(b->binding_name[0].kind, kind) == 0) {
+
+	  bi->destroy();
+
+	  if (b->binding_type == CosNaming::ncontext) {
+	    cerr << "Error: unbind: can't unbind a naming context.\n"
+		 << "Use remove_context to remove an empty naming context."
+		 << endl;
+	    exit(1);
+	  }
+
+	  context->unbind(b->binding_name);
+	  goto done;
+	}
+
+      }
+      bi->destroy();
+
+      cerr << "Error: unbind: couldn't find binding" << endl;
+      return 1;
     }
 
 
@@ -321,14 +401,19 @@ main(int argc, char **argv)
 
     if (strcmp(command, "resolve") == 0) {
 
-      CosNaming::Name name;
-      GetName(argc, argv, n, 0, "", name);
+      if (argc != n+1) {
+	cerr << "usage: " << argv[0] << " " << command
+	     << " name1_id.kind/.../nameN_id.kind" << endl;
+	exit(1);
+      }
+
+      CosNaming::Name_var name = omniURI::stringToName(argv[n]);
 
       CORBA::Object_var obj = rootContext->resolve(name);
 
       char* p = orb->object_to_string(obj);
       cout << p << endl;
-      return 0;
+      goto done;
     }
 
 
@@ -349,8 +434,13 @@ main(int argc, char **argv)
 
     if (strcmp(command, "bind_context") == 0) {
 
-      CosNaming::Name name;
-      GetName(argc, argv, n, 1, "objref", name);
+      if (argc != n+2) {
+	cerr << "usage: " << argv[0] << " " << command
+	     << " name1_id.kind/.../nameN_id.kind objref" << endl;
+	exit(1);
+      }
+
+      CosNaming::Name_var name = omniURI::stringToName(argv[n++]);
 
       CORBA::Object_var obj;
 
@@ -371,7 +461,7 @@ main(int argc, char **argv)
       }
 
       rootContext->bind_context(name, context);
-      return 0;
+      goto done;
     }
 
 
@@ -382,8 +472,13 @@ main(int argc, char **argv)
 
     if (strcmp(command, "rebind") == 0) {
 
-      CosNaming::Name name;
-      GetName(argc, argv, n, 1, "objref", name);
+      if (argc != n+2) {
+	cerr << "usage: " << argv[0] << " " << command
+	     << " name1_id.kind/.../nameN_id.kind objref" << endl;
+	exit(1);
+      }
+
+      CosNaming::Name_var name = omniURI::stringToName(argv[n++]);
 
       CORBA::Object_var obj;
 
@@ -396,7 +491,7 @@ main(int argc, char **argv)
       }
 
       rootContext->rebind(name, obj);
-      return 0;
+      goto done;
     }
 
 
@@ -407,8 +502,13 @@ main(int argc, char **argv)
 
     if (strcmp(command, "rebind_context") == 0) {
 
-      CosNaming::Name name;
-      GetName(argc, argv, n, 1, "objref", name);
+      if (argc != n+2) {
+	cerr << "usage: " << argv[0] << " " << command
+	     << " name1_id.kind/.../nameN_id.kind objref" << endl;
+	exit(1);
+      }
+
+      CosNaming::Name_var name = omniURI::stringToName(argv[n++]);
 
       CORBA::Object_var obj;
 
@@ -429,7 +529,7 @@ main(int argc, char **argv)
       }
 
       rootContext->rebind_context(name, context);
-      return 0;
+      goto done;
     }
 
 
@@ -449,7 +549,7 @@ main(int argc, char **argv)
 
       char* p = orb->object_to_string(context);
       cout << p << endl;
-      return 0;
+      goto done;
     }
 
 
@@ -466,7 +566,7 @@ main(int argc, char **argv)
       }
 
       rootContext->destroy();
-      return 0;
+      goto done;
     }
 
     usage(argv[0]);
@@ -482,33 +582,33 @@ main(int argc, char **argv)
       cerr << "not object";
     }
     cerr << endl;
-    return 1;
+    goto error_return;
   }
 
   catch (CosNaming::NamingContext::InvalidName) {
     cerr << command << ": InvalidName exception" << endl;
-    return 1;
+    goto error_return;
   }
 
   catch (CosNaming::NamingContext::AlreadyBound) {
     cerr << command << ": AlreadyBound exception" << endl;
-    return 1;
+    goto error_return;
   }
 
   catch (CosNaming::NamingContext::NotEmpty) {
     cerr << command << ": NotEmpty exception" << endl;
-    return 1;
+    goto error_return;
   }
 
   catch (CosNaming::NamingContext::CannotProceed) {
     cerr << command << ": CannotProceed exception" << endl;
-    return 1;
+    goto error_return;
   }
   catch (CORBA::COMM_FAILURE) {
     cerr << command 
 	 << ": Cannot contact the Naming Service because of COMM_FAILURE."
 	 << endl;
-    return 1;
+    goto error_return;
   }
 
   catch (...) {
@@ -518,30 +618,11 @@ main(int argc, char **argv)
   cerr << command
        << ": Unexpected error encountered."
        << endl;
+ error_return:
+  orb->destroy();
   return 1;
-}
 
-
-
-static void
-GetName(int argc, char** argv, int& n, int argsAfter, const char* usageAfter,
-	CosNaming::Name& name)
-{
-  int n_args = (argc - n - argsAfter);
-
-  if (((n_args % 2) != 0) || (n_args < 2)) {
-    cerr << "usage: " << argv[0] << " " << command
-	 << " name1_id name1_kind ... nameN_id nameN_kind " << usageAfter
-	 << endl;
-    exit(1);
-  }
-
-  int n_components = n_args / 2;
-
-  name.length(n_components);
-
-  for (int i = 0; i < n_components; i++) {
-    name[i].id   = (const char*) argv[n++];
-    name[i].kind = (const char*) argv[n++];
-  }
+ done:
+  orb->destroy();
+  return 0;
 }
