@@ -1,7 +1,7 @@
 //				Package : omnithread
 // omnithread/solaris.cc	Created : 7/94 tjr
 //
-//    Copyright (C) 1994,1995,1996, 1997 Olivetti & Oracle Research Laboratory
+//    Copyright (C) 1994-1999 AT&T Laboratories Cambridge
 //
 //    This file is part of the omnithread library
 //
@@ -95,11 +95,15 @@ omni_condition::timedwait(unsigned long secs, unsigned long nanosecs)
 {
     timespec rqts = { secs, nanosecs };
 
+ again:
     int rc = cond_timedwait(&sol_cond, &mutex->sol_mutex, &rqts);
 
     if (rc == 0)
 	return 1;
 
+    if (rc == EINTR)
+        goto again;
+
     if (rc == ETIME)
 	return 0;
 
@@ -169,6 +173,7 @@ int omni_thread::next_id = 0;
 
 static thread_key_t self_key;
 
+static size_t stack_size = 0;
 
 //
 // Initialisation function (gets called before any user code).
@@ -324,7 +329,7 @@ omni_thread::start(void)
     if (_state != STATE_NEW)
 	throw omni_thread_invalid();
 
-    THROW_ERRORS(thr_create(0, 0, omni_thread_wrapper, (void*)this, flags,
+    THROW_ERRORS(thr_create(0, stack_size, omni_thread_wrapper, (void*)this, flags,
 			    &sol_thread));
 
     _state = STATE_RUNNING;
@@ -493,8 +498,16 @@ void
 omni_thread::sleep(unsigned long secs, unsigned long nanosecs)
 {
     timespec rqts = { secs, nanosecs };
-    if (nanosleep(&rqts, (timespec*)NULL) != 0)
+    timespec remain;
+    while (nanosleep(&rqts, &remain)) {
+      if (errno == EINTR) {
+	rqts.tv_sec  = remain.tv_sec;
+	rqts.tv_nsec = remain.tv_nsec;
+	continue;
+      }
+      else
 	throw omni_thread_fatal(errno);
+    }
 }
 
 
@@ -529,3 +542,16 @@ omni_thread::sol_priority(priority_t pri)
 
     throw omni_thread_invalid();
 }
+
+
+void
+omni_thread::stacksize(unsigned long sz)
+{
+  stack_size = sz;
+}
+
+unsigned long
+omni_thread::stacksize()
+{
+  return stack_size;
+}
